using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Threading;


/// <summary>
/// Action event delegate
/// </summary>
public delegate void AsyncAction();


/// <summary>
/// Async worker status enumeration
/// </summary>
public enum AsyncWorkerStatusEnum : int
{
    /// <summary>
    /// Process stopped
    /// </summary>
    Stopped = 0,
    /// <summary>
    /// Process running
    /// </summary>
    Running = 1,
    /// <summary>
    /// Process finished
    /// </summary>
    Finished = 2,
    /// <summary>
    /// Error occured
    /// </summary>
    Error = 3
}


/// <summary>
/// Worker class for asynchronous operations
/// </summary>
public class AsyncWorker
{
    /// <summary>
    /// Worker status
    /// </summary>
    private AsyncWorkerStatusEnum mStatus = AsyncWorkerStatusEnum.Stopped;

    /// <summary>
    /// Action to run
    /// </summary>
    private AsyncAction mAction = null;

    /// <summary>
    /// Async thread
    /// </summary>
    private Thread mAsyncThread = null;

    /// <summary>
    /// Last exception
    /// </summary>
    private Exception mLastException = null;


    /// <summary>
    /// Worker status
    /// </summary>
    public AsyncWorkerStatusEnum Status
    {
        get
        {
            return mStatus;
        }
    }


    /// <summary>
    /// Last exception
    /// </summary>
    public Exception LastException
    {
        get
        {
            return mLastException;
        }
    }


    /// <summary>
    /// Resets the worker
    /// </summary>
    public void Reset()
    {
        if (this.Status == AsyncWorkerStatusEnum.Running)
        {
            throw new Exception("[AsyncWorker.RunAsync]: Cannot reset running action.");
        }
        mStatus = AsyncWorkerStatusEnum.Error;
        mAction = null;
        mAsyncThread = null;
        mLastException = null;
    }


    /// <summary>
    /// Stops the process
    /// </summary>
    public void Stop()
    {
        if (mAsyncThread != null)
        {
            mAsyncThread.Abort();
            mAsyncThread = null;
        }

        mStatus = AsyncWorkerStatusEnum.Stopped;
    }


    /// <summary>
    /// Runs the action in an asynchronous thread
    /// </summary>
    /// <param name="action">Action callback to run</param>
    public void RunAsync(AsyncAction action)
    {
        if (this.Status == AsyncWorkerStatusEnum.Running)
        {
            throw new Exception("[AsyncWorker.RunAsync]: Worker is currently running.");
        }
        mStatus = AsyncWorkerStatusEnum.Running;
        mAction = action;

        mAsyncThread = new Thread(new ThreadStart(Run));
        mAsyncThread.Start();
    }


    /// <summary>
    /// Runs the action
    /// </summary>
    private void Run()
    {
        try
        {
            mAction();

            // Call finished handler
            if (OnFinished != null)
            {
                OnFinished(this, new EventArgs());
            }

            mStatus = AsyncWorkerStatusEnum.Finished;
        }
        catch (Exception ex)
        {
            mStatus = AsyncWorkerStatusEnum.Error;
            mLastException = ex;

            // Call error handler
            if (OnError != null)
            {
                OnError(this, new EventArgs());
            }
        }
        finally
        {
            mAsyncThread = null;
        }
    }


    /// <summary>
    /// On error event handler
    /// </summary>
    public event EventHandler OnError;


    /// <summary>
    /// On finihed event handler
    /// </summary>
    public event EventHandler OnFinished;
}
