using System.IO;
using System.Data.SqlClient;
using System.Xml;
using System;
using System.Collections;
using System.Configuration;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.Configuration;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;

using CMS.GlobalHelper;
using CMS.SettingsProvider;
using CMS.CMSHelper;
using CMS.IDataConnectionLibrary;

/// <summary>
/// Enumeration of the SQL authentication mode
/// </summary>
public enum SQLServerAuthenticationModeEnum
{
    SQLServerAuthentication = 0,
    WindowsAuthentication = 1,
}


/// <summary>
/// Installer methods
/// </summary>
public static class InstallerFunctions
{
    /// <summary>
    /// Checks if the connection string is set
    /// </summary>
    /// <returns></returns>
    public static bool ConnectionStringIsSet()
    {
        return SqlHelperClass.IsConnectionStringInitialized;
    }


    /// <summary>
    /// Redirects user to the installation page if connectionString not set.
    /// </summary>
    public static bool InstallRedirect()
    {
        string currentPath = "";

        if (HttpContext.Current != null)
        {
            currentPath = HttpContext.Current.Request.Path.ToLower();
        }

        string appPath = "";

        if (HttpContext.Current != null)
        {
            appPath = HttpContext.Current.Request.ApplicationPath;
        }
        appPath = appPath == "/" ? "" : appPath;
        string relativePath = currentPath.Remove(0, appPath.Length);

        string currentFile = Path.GetFileName(relativePath);
        string fileExtension = Path.GetExtension(currentFile);

        if (currentFile == "install.aspx")
        {
            return true;
        }

        if (!ConnectionStringIsSet())
        {
            if (fileExtension == ".aspx")
            {
                if (!IsExcluded(relativePath))
                {
                    if (HttpContext.Current != null)
                    {
                        HttpContext.Current.Response.Redirect("~/cmsinstall/install.aspx");
                    }
                }
            }
            return true;
        }
        else
        {
            return false;
        }
    }


    private static bool IsExcluded(string path)
    {
        if (path.StartsWith("/cmsmessages"))
        {
            return true;
        }
        return false;
    }


    ///<summary>Checks if all folders of the given path exist and if not, it creates them.</summary>
    ///<param name="path">Full disk path.</param>
    public static void EnsureDiskPath(string path)
    {
        // Get the folder path
        string folderPath = Path.GetDirectoryName(path);
        // Create the directory
        if (!Directory.Exists(folderPath))
        {
            Directory.CreateDirectory(folderPath);
        }
    }


    /// <summary>
    /// Tests the given connection parameters
    /// </summary>
    /// <param name="authenticationMode">Authentication type</param>
    /// <param name="serverName">Server name</param>
    /// <param name="databaseName">Database name</param>
    /// <param name="userName">User name</param>
    /// <param name="password">User password</param>
    public static string TestConnection(SQLServerAuthenticationModeEnum authenticationMode, string serverName, string databaseName, string userName, string password)
    {
        try
        {
            string connectionString = GetConnectionString(authenticationMode, serverName, databaseName, userName, password, 10, false);
            IDataConnection conn = SqlHelperClass.GetConnection(connectionString);
            conn.Open();
            conn.Close();

            return "";
        }
        catch (Exception ex)
        {
            return ex.Message;
        }
    }


    /// <summary>
    /// Checks if the specified database exists, returns true if database exists
    /// </summary>
    /// <param name="authenticationMode">Authentication type</param>
    /// <param name="serverName">Server name</param>
    /// <param name="databaseName">Database name</param>
    /// <param name="userName">User name</param>
    /// <param name="password">User password</param>
    public static bool DatabaseExists(SQLServerAuthenticationModeEnum authenticationMode, string serverName, string databaseName, string userName, string password)
    {
        // Prepare the query
        string connectionString = GetConnectionString(authenticationMode, serverName, "master", userName, password, 120, false);
        string query = "SELECT Count(name) FROM sysdatabases WHERE name = '" + databaseName.Replace("'", "''") + "'";
        IDataConnection conn = SqlHelperClass.GetConnection(connectionString);

        // Get the databases count
        DataSet ds = conn.ExecuteQuery(query, null, QueryTypeEnum.SQLQuery);
        if ( !DataHelper.DataSourceIsEmpty(ds) )
        {
            return (ValidationHelper.GetInteger(ds.Tables[0].Rows[0][0], 0) > 0);
        }
        return false;
    }


    /// <summary>
    /// Generates the connection string with the specified parameters
    /// </summary>
    /// <param name="authenticationMode">Authentication type</param>
    /// <param name="serverName">Server name</param>
    /// <param name="databaseName">Database name</param>
    /// <param name="userName">User name</param>
    /// <param name="password">User password</param>
    /// <param name="timeout">Connection timeout</param>
    /// <param name="htmlEncoded">Html encoded to be able to diplay it properly on the page</param>
    public static string GetConnectionString(SQLServerAuthenticationModeEnum authenticationMode, string serverName, string databaseName, string userName, string password, int timeout, bool htmlEncoded)
    {
        password = htmlEncoded ? HttpUtility.HtmlEncode(HttpUtility.HtmlEncode(password)) : password;

        string result = null;
        if (authenticationMode == SQLServerAuthenticationModeEnum.SQLServerAuthentication)
        {
            // connection string for SQL Server authentication
            result = "Persist Security Info=False;database=" + databaseName + ";server=" + serverName + ";user id=" + userName + ";password=" + password + ";Current Language=English";
        }
        else
        {
            // connection string for Windows authentication
            result = "Persist Security Info=False;Integrated Security=SSPI;database=" + databaseName + ";server=" + serverName + ";Current Language=English";
        }
        // Add timeout parameter
        if (timeout > 0)
        {
            result += ";Connection Timeout=" + timeout + ";";
        }

        return result;
    }


    /// <summary>
    /// Performs the string replacement within the file
    /// </summary>
    /// <param name="filePath">File path</param>
    /// <param name="what">String to replace (old string)</param>
    /// <param name="replaceWith">Replacement (new string)</param>
    public static void ReplaceInFile(string filePath, string what, string replaceWith)
    {
        ReplaceInFile(filePath, what, replaceWith, null);
    }


    /// <summary>
    /// Performs the string replacement within the file
    /// </summary>
    /// <param name="filePath">File path</param>
    /// <param name="what">String to replace (old string)</param>
    /// <param name="replaceWith">Replacement (new string)</param>
    /// <param name="encoding">File encoding</param>
    public static void ReplaceInFile(string filePath, string what, string replaceWith, System.Text.Encoding encoding)
    {
        // Read the file
        System.IO.StreamReader str = null;
        string txt = null;
        StreamWriter sw = null;
        if (encoding == null)
        {
            encoding = System.Text.Encoding.Default;
        }
        str = File.OpenText(filePath);
        txt = str.ReadToEnd();
        str.Close();
        // Perform the replacement
        txt = txt.Replace(what, replaceWith);
        // Write the file back
        File.Delete(filePath);
        sw = new StreamWriter(filePath, false, encoding);
        sw.Write(txt);
        sw.Close();
    }


    /// <summary>
    /// Sets or adds the specified key to the AppSettings section
    /// </summary>
    /// <param name="configFilePath">Path to the configuration file</param>
    /// <param name="key">Settings key</param>
    /// <param name="value">Settings value</param>
    public static bool SetConfigValue(string configFilePath, string key, string value)
    {
        try
        {
            Configuration config = WebConfigurationManager.OpenWebConfiguration("~");

            if (config.AppSettings.Settings[key] != null)
            {
                config.AppSettings.Settings[key].Value = value;
            }
            else
            {
                config.AppSettings.Settings.Add(key, value);
            }

            config.Save();
            return true;
        }
        catch
        {
            return false;
        }
    }



    /// <summary>
    /// Sets or adds the specified connection string in the ConnectionStrings section
    /// </summary>
    /// <param name="configFilePath">Path to the configuration file</param>
    /// <param name="name">ConnectionString name</param>
    /// <param name="connectionString">Connection string</param>
    public static bool SetConnectionString(string configFilePath, string name, string connectionString)
    {
        try
        {
            Configuration config = WebConfigurationManager.OpenWebConfiguration("~");

            if (config.ConnectionStrings.ConnectionStrings[name] != null)
            {
                config.ConnectionStrings.ConnectionStrings[name].ConnectionString = connectionString;
            }
            else
            {
                config.ConnectionStrings.ConnectionStrings.Add(new ConnectionStringSettings(name, connectionString));
            }

            config.Save(ConfigurationSaveMode.Modified);
            return true;
        }
        catch
        {
            return false;
        }
    }
}

