using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Threading;

using CMS.SiteProvider;
using CMS.CMSHelper;
using CMS.PortalEngine;
using CMS.SettingsProvider;
using CMS.WebAnalytics;
using CMS.TreeEngine;

/// <summary>
/// Base interface for document pages
/// </summary>
public class DocumentBase
{
    #region "Variables"

    private string mCacheItems = "username;viewmode;sitename;lang"; //domain

    private CurrentSiteInfo mCurrentSite = null;
    private PageInfo mCurrentPage = null;
    private CurrentUserInfo mCurrentUser = null;

    private HttpResponse mResponse = null;

    #endregion


    #region "Public properties"

    /// <summary>
    /// Current site
    /// </summary>
    public CurrentSiteInfo CurrentSite
    {
        get
        {
            if (mCurrentSite == null)
            {
                mCurrentSite = CMSContext.CurrentSite;
            }
            return mCurrentSite;
        }
    }


    /// <summary>
    /// Current user
    /// </summary>
    public CurrentUserInfo CurrentUser
    {
        get
        {
            if (mCurrentUser == null)
            {
                mCurrentUser = CMSContext.CurrentUser;
            }
            return mCurrentUser;
        }
    }


    /// <summary>
    /// Current page
    /// </summary>
    public PageInfo CurrentPage
    {
        get
        {
            if (mCurrentPage == null)
            {
                mCurrentPage = CMSContext.CurrentPageInfo;
            }
            return mCurrentPage;
        }
    }


    /// <summary>
    /// List of the cache key items
    /// </summary>
    public string CacheItems
    {
        get
        {
            return mCacheItems;
        }
        set
        {
            mCacheItems = value;
        }
    }


    /// <summary>
    /// Current response
    /// </summary>
    public HttpResponse Response
    {
        get
        {
            if (mResponse == null)
            {
                mResponse = HttpContext.Current.Response;
            }
            return mResponse;
        }
    }

    #endregion


    #region "Methods"

    /// <summary>
    /// Sets the current thread culture acoording to the user culture
    /// </summary>
    public void SetCulture()
    {
        // Set the culture
        if ((this.CurrentUser.PreferredCultureCode != null) && (this.CurrentUser.PreferredCultureCode != ""))
        {
            System.Globalization.CultureInfo ci = new System.Globalization.CultureInfo(this.CurrentUser.PreferredCultureCode);
            Thread.CurrentThread.CurrentCulture = ci;
            Thread.CurrentThread.CurrentUICulture = ci;
        }
    }


    /// <summary>
    /// Sets the page caching
    /// </summary>
    public void SetCaching()
    {
        // Cache control
        int cacheMinutes = this.CurrentPage.NodeCacheMinutes;
        if (cacheMinutes < 0)
        {
            cacheMinutes = SettingsKeyProvider.GetIntValue(this.CurrentSite.SiteName + ".CMSCacheMinutes");
        }
        if ((cacheMinutes > 0) && (this.CurrentUser.ViewMode == ViewModeEnum.LiveSite))
        {
            Response.Cache.SetCacheability(HttpCacheability.Server);
            Response.Cache.SetValidUntilExpires(true);
            Response.Cache.SetExpires(DateTime.Now.AddMinutes(cacheMinutes));
            Response.Cache.VaryByParams["*"] = true;
            Response.Cache.SetVaryByCustom(CacheItems); //domain
        }
        else
        {
            Response.Cache.SetNoStore();
        }

        Response.Cache.SetRevalidation(HttpCacheRevalidation.AllCaches);
    }


    /// <summary>
    /// Sets the campaign parameter if present
    /// </summary>
    public void SetCampaign()
    {
        if (CurrentUser.ViewMode == ViewModeEnum.LiveSite)
        {
            // Store campaign name if present
            string campaign = AnalyticsHelper.CurrentCampaign(CurrentSite.SiteName);
            if (campaign != "")
            {
                CurrentUser.Campaign = campaign;

                // Log campaign
                if (AnalyticsHelper.AnalyticsEnabled(CurrentSite.SiteName) && !AnalyticsHelper.IsURLExcluded(CurrentSite.SiteName, CurrentPage.NodeAliasPath) && !AnalyticsHelper.IsIPExcluded(CurrentSite.SiteName, HttpContext.Current.Request.UserHostAddress) && !AnalyticsHelper.IsSearchEngineExcluded(CurrentSite.SiteName))
                {
                    HitLogProvider.LogHit(HitLogProvider.CAMPAIGNS, CurrentSite.SiteName, null, campaign, 0);
                }
            }
        }
    }


    /// <summary>
    /// Logs the page hit
    /// </summary>
    public void LogHit()
    {
        // Log the page view hit in live site mode
        if (CurrentUser.ViewMode == ViewModeEnum.LiveSite)
        {
            if (AnalyticsHelper.AnalyticsEnabled(CurrentSite.SiteName) && !AnalyticsHelper.IsURLExcluded(CurrentSite.SiteName, CurrentPage.NodeAliasPath) && !AnalyticsHelper.IsIPExcluded(CurrentSite.SiteName, HttpContext.Current.Request.UserHostAddress) && !AnalyticsHelper.IsSearchEngineExcluded(CurrentSite.SiteName))
            {
                HitLogProvider.LogHit(HitLogProvider.PAGE_VIEWS, CurrentSite.SiteName, CurrentUser.PreferredCultureCode, "", CurrentPage.NodeId);
            }
        }
    }

    
    /// <summary>
    /// Adds the CMS page tags
    /// </summary>
    /// <param name="page">Page</param>
    public void AddCMSTags(Page page)
    {
        // Language tag
        string tags = "<input type=\"hidden\" name=\"lng\" id=\"lng\" value=\"" + CurrentUser.PreferredCultureCode + "\" />\n";
        if (CurrentPage != null)
        {
            // View mode tag
            if (TreePathUtils.IsMenuItemType(CurrentPage.ClassName))
            {
                if (CurrentUser.MenuItemViewMode != ViewModeEnum.LiveSite)
                {
                    tags += "<input type=\"hidden\" name=\"mvmode\" id=\"mvmode\" value=\"" + ViewModeCode.FromEnum(CurrentUser.MenuItemViewMode) + "\" />\n";
                }
            }
            else
            {
                if (CurrentUser.ViewMode != ViewModeEnum.LiveSite)
                {
                    tags += "<input type=\"hidden\" name=\"vmode\" id=\"vmode\" value=\"" + ViewModeCode.FromEnum(CurrentUser.ViewMode) + "\" />\n";
                }
            }
        }

        page.ClientScript.RegisterClientScriptBlock(typeof(string), "CMSTags", tags);
    }

    #endregion

};
