using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Drawing;
using System.Collections.Generic;
using System.IO;

using ZedGraph;
using ZedGraph.Web;

using CMS.CMSHelper;
using CMS.GlobalHelper;
using CMS.Reporting;
using System.Security.Cryptography;


/// <summary>
/// Summary description for ReportGraph
/// </summary>
public class ReportGraph
{
    public const int DEFAULT_WIDTH = 400;
    public const int DEFAULT_HEIGHT = 250;
    public const int NO_LEGEND = -1;

    private DataSet mDataSet = null;
    private ZedGraphWeb zg1 = null;
    private string mGraphTitle = "";
    private string mXAxisTitle = "";
    private string mYAxisTitle = "";
    private GraphTypeEnum mGraphType = GraphTypeEnum.Bar;
    private Color mGraphBackgroundColor = Color.FromArgb(200, 200, 255);
    private Color mGraphChartBackgroundColor = Color.LightGoldenrodYellow;
    private LegendPos mLegendPosition = LegendPos.Right;
    private bool mNoLegend = false;
    private bool mGenerateInvalidDataGraph = false;    

    //standard colors, first in use, when is more items then standard colors, random color is generated
    protected static Color[] standardBarColors = new Color[] { Color.Navy, Color.Purple, Color.LimeGreen, Color.SandyBrown, Color.Red, Color.Blue, Color.Green, Color.Yellow, Color.Black };

    //public BarColors-can be manipulated from outside, filled with standardbarColors in constuctor
    public List<Color> BarsColors = new List<Color>();


    #region Public properties
    /// <summary>
    /// Data source
    /// </summary>
    public DataSet DataSource
    {
        get
        {
            return mDataSet;
        }
        set
        {
            mDataSet = value;
        }
    }

    /// <summary>
    /// The title on output graph
    /// </summary>
    public string GraphTitle
    {
        get { return mGraphTitle; }
        set { mGraphTitle = value; }
    }


    /// <summary>
    /// The graph type
    /// </summary>
    public GraphTypeEnum GraphType
    {
        get { return mGraphType; }
        set { mGraphType = value; }
    }


    /// <summary>
    /// The title of x-axis on output graph
    /// </summary>
    public string xAxisTitle
    {
        get { return mXAxisTitle; }
        set { mXAxisTitle = value; }
    }


    /// <summary>
    /// The title of y-axis on output graph
    /// </summary>
    public string yAxisTitle
    {
        get { return mYAxisTitle; }
        set { mYAxisTitle = value; }
    }


    /// <summary>
    /// Graph backgorund color
    /// </summary>
    public Color GraphBackgroundColor
    {
        get { return mGraphBackgroundColor; }
        set { mGraphBackgroundColor = value; }
    }


    /// <summary>
    /// Graph chart backgorund color
    /// </summary>
    public Color GraphChartBackgroundColor
    {
        get { return mGraphChartBackgroundColor; }
        set { mGraphChartBackgroundColor = value; }
    }


    /// <summary>
    /// Legend position
    /// </summary>
    public LegendPos LegendPosition
    {
        get { return mLegendPosition; }
        set { mLegendPosition = value; }
    }


    /// <summary>
    /// Sets whether legend will be dispalyed
    /// </summary>
    public bool NoLegend
    {
        get { return mNoLegend; }
        set { mNoLegend = value; }
    }



    /// <summary>
    /// Width of graph
    /// </summary>
    public int Width
    {
        get { return zg1.Width; }
        set { zg1.Width = (value<=0)?DEFAULT_WIDTH:value; }
    }


    /// <summary>
    /// Height of graph
    /// </summary>
    public int Height
    {
        get { return zg1.Height; }
        set { zg1.Height = (value <= 0) ? DEFAULT_HEIGHT : value; }
    }


    #endregion


    /// <summary>
    /// Creates a graph from previously set datasource
    /// </summary>
    /// <returns>byte array</returns>
    public byte[] CreateGraph()
    {
        MemoryStream stream = new MemoryStream();
        this.zg1.Draw(stream);
        return stream.ToArray();        
    }


    /// <summary>
    /// Creates a graph image data
    /// </summary>
    /// <returns>byte array</returns>
    public byte[] CreateGraph(ReportGraphInfo reportGraph, DataSet ds)
    {
        //reportGraph.GraphReportID
        if (!DataHelper.DataSourceIsEmpty(ds))
        {
            GraphTypeEnum gte = GraphTypeEnum.Bar;

            if (reportGraph.GraphType.ToLower() == "pie")
            {
                gte = GraphTypeEnum.Pie;
            }

            this.DataSource = ds;

            this.GraphType = gte;

            //graph legend settings
            if (reportGraph.GraphLegendPosition == ReportGraph.NO_LEGEND)
            {
                this.NoLegend = true;
            }
            else
            {
                this.LegendPosition = (LegendPos)reportGraph.GraphLegendPosition;
            }

            this.xAxisTitle = reportGraph.GraphXAxisTitle;
            this.yAxisTitle = reportGraph.GraphYAxisTitle;
            this.GraphTitle = reportGraph.GraphTitle;
            this.Height = reportGraph.GraphHeight;
            this.Width = reportGraph.GraphWidth;

            return CreateGraph();
        }
        else //datasourceempty, show very small image - don't want it to be visible
        {
            this.Width = 1;
            this.Height = 1;
            return CreateGraph();
        }
    }

    /// <summary>
    /// Creates a graph for invalid data, bad datasets, incorrect format ..
    /// </summary>
    /// <returns>byte array</returns>
    public byte[] CreateInvalidGraph(ReportGraphInfo reportGraph)
    {

        mGenerateInvalidDataGraph = true;

        this.xAxisTitle = reportGraph.GraphXAxisTitle;
        this.yAxisTitle = reportGraph.GraphYAxisTitle;        
        this.Height = reportGraph.GraphHeight;
        this.Width = reportGraph.GraphWidth;

        MemoryStream stream = new MemoryStream();
        this.zg1.Draw(stream);
        return stream.ToArray();

    }


    /// <summary>
    /// Required method for Designer support - do not modify
    /// the contents of this method with the code editor.
    /// </summary>
    private void InitializeComponent()
    {
        this.zg1.RenderGraph += new ZedGraph.Web.ZedGraphWebControlEventHandler(this.OnRenderGraph);
    }

    /// <summary>
    /// This method is where you generate your graph.
    /// </summary>
    /// <param name="masterPane">You are provided with a MasterPane instance that
    /// contains one GraphPane by default (accessible via masterPane[0]).</param>
    /// <param name="g">A graphics instance so you can easily make the call to AxisChange()</param>
    private void OnRenderGraph(ZedGraphWeb zgw, System.Drawing.Graphics g, ZedGraph.MasterPane masterPane)
    {
        if (mGenerateInvalidDataGraph)
        {
            //sets the invalid graph 
            InvalidDataGraph(zgw, g, masterPane);
            return;
        }

        ContextResolver resolver = CMSContext.CurrentResolver.CreateChild();
        DataRow[] dRows = new DataRow[1];
        dRows[0] = CMSContext.GetContextParametersDataRow(); ;
        object[,] mAllParameters = CMS.DataEngine.SpecialFunctions.ConvertDataRowsToParams(dRows, null);

        resolver.SourceParameters = mAllParameters;

        DataSet ds = this.DataSource;
        if (!DataHelper.DataSourceIsEmpty(ds))
        {
            // Get the GraphPane so we can work with it
            GraphPane myPane = masterPane[0];

            myPane.Title.Text = this.GraphTitle;
            myPane.Title.FontSpec.Size = 24f;

            myPane.Fill = new Fill(Color.White, this.GraphBackgroundColor, 45.0f);
            myPane.Chart.Fill = new Fill(Color.White, this.GraphChartBackgroundColor, 45.0f);

            if (this.NoLegend)
            {
                myPane.Legend.IsVisible = false;
            }
            else
            {
                myPane.Legend.Position = this.LegendPosition;
            }

            if (GraphType == GraphTypeEnum.Bar) //classic vertical bar graph
            {
                //if x-title is not explicitly set use name of first column
                myPane.XAxis.Title.Text = this.xAxisTitle;
                myPane.YAxis.Title.Text = this.yAxisTitle;

                List<string> columnNamesList = new List<string>();
                List<string> firstColumnNames = new List<string>();
                List<PointPairList> valuesList = new List<PointPairList>();


                //get data from data source, in first column are categories, next columns are values - it can be more
                // values for each category - multiple bars, and column names are descriptions for values
                // Point is a need to go trough all data set because a have values in columns and need values from all rows.                
                double maxYValue = 0;
                int columnCount = ds.Tables[0].Columns.Count;
                if (columnCount > 1)
                {
                    for (int i = 1; i < columnCount; i++)
                    {
                        valuesList.Add(new PointPairList()); //create list for all columns with datas
                        columnNamesList.Add(ds.Tables[0].Columns[i].ColumnName);
                    }

                    int rowOrder = 0;
                    foreach (DataRow row in ds.Tables[0].Rows)
                    {
                        rowOrder++;
                        firstColumnNames.Add( resolver.ResolveMacros(ValidationHelper.GetString(row[0], "")));
                        for (int j = 1; j < columnCount; j++)
                        {
                            double value = ValidationHelper.GetDouble(row[j], 0);
                            valuesList[j - 1].Add(rowOrder, value);
                            maxYValue = (value > maxYValue) ? value : maxYValue; //get max-y value for labels above bars
                        }
                    }
                }

                //labels above bars
                float shift = (float)(0.03f*maxYValue); //how much above bar labels will be - 3% of biggest bar
                float columns = columnCount - 1;                
                float colSpace = (1f / (columns + 1 + (columns - 1) * 0.25f));                

                for (int i = 0; i < columnCount - 1; i++)
                {
                    float colOrder = (float)(columnCount-1-i);
                    int labelOrder = 0;

                    foreach (PointPair pair in valuesList[i])
                    {
                        //don't create labels for zero values
                        //if (pair.Y == 0)
                        //{
                        //    labelOrder++;
                        //    continue;
                        //}

                        // format the label string to have 1 decimal place
                        string lab = pair.Y.ToString("F0");
                        // get exact x coordinate where to draw label                        
                        float xcoord = (float)(1.5 + labelOrder - (colSpace / 2) - (colOrder - 1) * (0.25 * colSpace) - (colSpace * (colOrder - 0.5)));
                        // for negative bars, the label appears just above the zero value                        
                        TextObj text = new TextObj(lab, xcoord, (float)(pair.Y < 0 ? 0.0 : pair.Y) + shift);
                        // tell Zedgraph to use user scale units for locating the TextItem
                        text.Location.CoordinateFrame = CoordType.AxisXYScale;
                        // AlignH the left-center of the text to the specified point
                        text.Location.AlignH = AlignH.Left;
                        text.Location.AlignV = AlignV.Center;
                        text.FontSpec.Border.IsVisible = false;
                        text.FontSpec.Fill.IsVisible = false;
                        // rotate the text 90 degrees
                        text.FontSpec.Angle = 90;
                        // add the TextItem to the list
                        myPane.GraphObjList.Add(text);
                        labelOrder++;
                    }
                }

                for (int i = 0; i < columnCount - 1; i++)
                {
                    //choose bar color
                    Color color = (BarsColors.Count > i) ? BarsColors[i] : GetRandomColor();
                    //add bars to drawing pane
                    BarItem myCurve = myPane.AddBar(columnNamesList[i], valuesList[i], color);
                }

                myPane.XAxis.MajorTic.IsBetweenLabels = true;
                string[] labels = firstColumnNames.ToArray();
                myPane.XAxis.Scale.TextLabels = labels;
                myPane.XAxis.Scale.IsPreventLabelOverlap = false;
                myPane.XAxis.Type = AxisType.Text;
                //minimum value is 0 - webanalytics; bars with negative values won't be displayed correctly
                myPane.YAxis.Scale.Min = 0; 
                masterPane.AxisChange(g);
            }
            else // classic pie graph
            {
                // No fill for the chart background
                myPane.Chart.Fill.Type = FillType.None;

                //first column name, second value, third optional offset=indent from center
                if (ds.Tables[0].Columns.Count >= 2) //at least two columns are required
                {
                    int i = 0;
                    foreach (DataRow row in ds.Tables[0].Rows)
                    {
                        //get indent
                        double offset = 0.0;
                        if (ds.Tables[0].Columns.Count >= 3)
                        {
                            offset = ValidationHelper.GetDouble(row[2], 0.0);
                        }

                        //choose segment color
                        Color color = (BarsColors.Count > i) ? BarsColors[i] : GetRandomColor();

                        double value = Math.Round(ValidationHelper.GetDouble(row[1], 0), 4);
                        string segmentLabel = resolver.ResolveMacros(ValidationHelper.GetString(row[0], ""));
                        PieItem segment = myPane.AddPieSlice(value, color, Color.White, 45f, offset, segmentLabel);
                        segment.LabelType = PieLabelType.Name_Value;
                        i++;
                    }
                }
                else //invalid data source
                {
                    InvalidDataGraph(zgw, g, masterPane);
                }

                masterPane.AxisChange(g);
            }
        }
        else //generate no-data graph
        {
            InvalidDataGraph(zgw, g, masterPane);
        }
    }


    /// <summary>
    /// Sets graph for invalidData state
    /// </summary>
    /// <param name="zgw">graph</param>
    /// <param name="g"></param>
    /// <param name="masterPane"></param>
    private void InvalidDataGraph(ZedGraphWeb zgw, System.Drawing.Graphics g, ZedGraph.MasterPane masterPane)
    {
        // Get the GraphPane so we can work with it
        GraphPane myPane = masterPane[0];

        myPane.Title.Text = ResHelper.GetString("Reporting.Graph.InvalidDataGraph");
        myPane.Title.FontSpec.FontColor = Color.Red;
        myPane.Title.FontSpec.Size = 24f;

        myPane.XAxis.Title.Text = this.xAxisTitle;
        myPane.YAxis.Title.Text = this.yAxisTitle;

        myPane.XAxis.MajorTic.IsBetweenLabels = true;
        myPane.XAxis.Type = AxisType.Text;

        masterPane.AxisChange(g);
    }


    /// <summary>
    /// Basic constructor of graph
    /// </summary>
    public ReportGraph()
    {
        BarsColors.AddRange(standardBarColors);
        zg1 = new ZedGraphWeb();
        zg1.Width = DEFAULT_WIDTH;
        zg1.Height = DEFAULT_HEIGHT;
        InitializeComponent();
    }


    /// <summary>
    /// Constructor with width and height setting
    /// </summary>
    /// <param name="width">width of graph</param>
    /// <param name="height">height of graph</param>
    public ReportGraph(int width, int height)
    {
        BarsColors.AddRange(standardBarColors);
        zg1 = new ZedGraphWeb();
        zg1.Width = width;
        zg1.Height = height;
        InitializeComponent();
    }


    /// <summary>
    /// To get random color
    /// </summary>
    /// <returns>Color</returns>
    public static Color GetRandomColor()
    {
        KnownColor[] colors = (KnownColor[])Enum.GetValues(typeof(KnownColor));
        return Color.FromKnownColor(colors[GetRandomNumber(colors.Length)]);
    }

    /// <summary>
    /// To get random number
    /// </summary>
    /// <param name="maxValue">maximal random number value</param>
    /// <returns>integer</returns>
    public static int GetRandomNumber(int maxValue)
    {
        RandomNumberGenerator rng = RNGCryptoServiceProvider.Create();
        byte[] bytes = new byte[4];
        rng.GetBytes(bytes);
        int ranNum = BitConverter.ToInt32(bytes, 0);
        return Math.Abs(ranNum % maxValue);
    }
}
