using System;
using System.Data;
using System.Data.SqlClient;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Xml;
using System.IO;

using CMS.DataEngine;
using CMS.GlobalHelper;
using CMS.FormEngine;
using CMS.ExtendedControls;

public partial class UniGrid : InlineUserControl
{
    /// <summary>
    /// Event raised on action postback
    /// </summary>
    public event OnActionEventHandler OnAction;
    /// <summary>
    /// Event raised when external source data required
    /// </summary>
    public event OnExternalDataBoundEventHandler OnExternalDataBound;
    /// <summary>
    /// Event raised when data are reloaded from external datasource.
    /// </summary>
    public event OnDataReload OnDataReload;

    private string gGridName;
    private string mWhereCondition;
    private object mDataSource = null;
    private string mImageDirectoryPath = null;
    private string mOrderBy = null;
    private object[,] mQueryParameters = null;
    private bool mHideControlForZeroRows = false;
    private string mZeroRowsText = "";


    #region "Public properties"

    /// <summary>
    /// Inline control parameter - mirrors the GridName property
    /// </summary>
    public override string Parameter
    {
        get
        {
            return this.GridName;
        }
        set
        {
            this.GridName = value;
        }
    }

    /// <summary>
    /// External grid DataSource, when set, overrides the default datasource settings
    /// </summary>
    public object DataSource
    {
        get
        {
            return mDataSource;
        }
        set
        {
            mDataSource = value;
        }
    }


    /// <summary>
    /// Xml file with source information
    /// </summary>
    public string GridName
    {

        get
        {
            return gGridName;
        }

        set
        {
            gGridName = value;
        }

    }

    /// <summary>
    /// Grid view used by UniGrid
    /// </summary>
    public GridView GridView
    {
        get
        {
            return UniGridView;
        }

        set
        {
            UniGridView = value;
        }

    }


    /// <summary>
    /// Where condition that can be set from other classes. 
    /// </summary>
    public String WhereCondition
    {
        get
        {
            return mWhereCondition;
        }

        set
        {
            mWhereCondition = value;
        }

    }


    /// <summary>
    /// Order columns 
    /// </summary>
    public string OrderBy
    {
        get
        {
            return mOrderBy;
        }
        set
        {
            mOrderBy = value;
            if (!this.IsPostBack)
            {
                SortDirect = value;
            }
        }
    }


    /// <summary>
    /// Query parameters for the query
    /// </summary>
    public object[,] QueryParameters
    {
        get
        {
            return mQueryParameters;
        }
        set
        {
            mQueryParameters = value;
        }
    }


    /// <summary>
    /// Gets or sets directory path for images
    /// </summary>
    public string ImageDirectoryPath
    {
        get
        {
            return mImageDirectoryPath;
        }
        set
        {
            if (!value.EndsWith("/"))
            {
                mImageDirectoryPath = value + "/";
            }
            else
            {
                mImageDirectoryPath = value;
            }
        }

    }


    /// <summary>
    /// Text displayed when unigrid datasource is empty.
    /// </summary>
    public string ZeroRowsText
    {

        get
        {
            return mZeroRowsText;
        }

        set
        {
            mZeroRowsText = value;
        }

    }


    /// <summary>
    /// Indicates whether hide unigrid or not when its datasource is empty.
    /// </summary>
    public bool HideControlForZeroRows
    {

        get
        {
            return mHideControlForZeroRows;
        }

        set
        {
            mHideControlForZeroRows = value;
        }

    }

    #endregion


    #region "Private properties"

    /// <summary>
    /// Where clause used in grid view
    /// </summary>
    private String WhereClause
    {
        get
        {
            object o = ViewState["WhereClause"];
            return (o == null) ? String.Empty : (string)o;
        }

        set
        {
            ViewState["WhereClause"] = value;
        }
    }


    /// <summary>
    /// Query used in grid view
    /// </summary>
    private String Query
    {
        get
        {
            object o = ViewState["Query"];
            return (o == null) ? String.Empty : (string)o;
        }

        set
        {
            ViewState["Query"] = value;
        }
    }


    /// <summary>
    /// Sort directive used in grid view
    /// </summary>
    private String SortDirect
    {
        get
        {
            object o = ViewState["SortDirect"];
            return (o == null) ? String.Empty : (string)o;
        }

        set
        {
            ViewState["SortDirect"] = value;
        }
    }


    /// <summary>
    /// Returns the complete where condition
    /// </summary>
    private string CompleteWhereCondition
    {
        get
        {
            string completeWhere = null;
            if ((WhereClause != null) && (WhereClause != ""))
            {
                completeWhere = WhereClause;
            }
            if ((WhereCondition != null) && (WhereCondition != ""))
            {
                if (completeWhere != null)
                {
                    completeWhere = "(" + completeWhere + ") AND (" + WhereCondition + ")";
                }
                else
                {
                    completeWhere = WhereCondition;
                }
            }
            return completeWhere;
        }
    }

    #endregion


    // is unigrid data source empty ?
    protected bool isEmpty = false;
    protected bool displayFilter = false;
    protected bool displayPageSize = false;

    /// <summary>
    /// Control load event handler
    /// </summary>
    protected void Page_Load(object sender, EventArgs e)
    {


        if (mImageDirectoryPath == null)
        {
            mImageDirectoryPath = "~/App_Themes/" + Page.Theme + "/Images/CMSAdminControls/UniGrid/";
        }

        lblItems.Text = ResHelper.GetString("UniGrid.ItemsPerPage");

        Table filterTable = new Table();
        XmlDocument document = new XmlDocument();
        document.Load(Server.MapPath(gGridName));
        XmlNode node = document.DocumentElement;

        UniGridView.Columns.Clear(); // clear all columns from the grid view
        UniGridView.GridLines = GridLines.Horizontal;

        XmlNode optionNode = node.SelectSingleNode("options");

        if (gGridName != null)
        {
            // Create filter table according to the key value "DisplayFilter"            
            displayFilter = (GetKeyValue(optionNode, "DisplayFilter").ToLower() == "true") ? true : false;
            if (displayFilter)
            {
                filter.Controls.Add(filterTable);
            }

            // Process 'template field'
            // --------------------------------------------------

            XmlNode actions = node.SelectSingleNode("actions");
            if (actions != null)
            {
                TemplateField ckhColumn = new TemplateField(); // custom template field of the grid view

                // fill in the custom template field
                ckhColumn.HeaderTemplate = new GridViewTemplate(ListItemType.Header, ResHelper.GetString("unigrid.actions"));
                GridViewTemplate actionsTemplate = new GridViewTemplate(ListItemType.Item, actions, ImageDirectoryPath);
                actionsTemplate.Page = this.Page;
                ckhColumn.ItemTemplate = actionsTemplate;

                ckhColumn.ItemStyle.Wrap = false;

                // add custom column to grid view
                UniGridView.Columns.Add(ckhColumn);

                actionsTemplate.OnExternalDataBound += new OnExternalDataBoundEventHandler(RaiseExternalDataBound);
            }


            // process 'bound fields'
            // --------------------------------------------------

            // select list of "column" nodes
            XmlNodeList columnList = node.SelectSingleNode("columns").SelectNodes("column");

            // get attributes AND filter node of each "column" node
            foreach (XmlNode columnNode in columnList)
            {

                string columnSource = null, columnCaption = null; // attributes of the "column" node
                DataControlField field = null;

                // Process the column type Hyperlink or BoundColumn based on the parameters
                if ((columnNode.Attributes["href"] != null) || (columnNode.Attributes["externalsourcename"] != null) || (columnNode.Attributes["localize"] != null))
                {
                    ExtendedBoundField linkColumn = new ExtendedBoundField();
                    field = (DataControlField)linkColumn;

                    // Attribute "source"
                    if (columnNode.Attributes["source"] != null)
                    {
                        columnSource = columnNode.Attributes["source"].Value;
                        linkColumn.DataField = columnSource;
                        if (columnSource.ToLower() != ExtendedBoundField.ALL_DATA.ToLower())
                        {
                            linkColumn.SortExpression = columnSource;
                        }
                    }

                    if (columnNode.Attributes["parameters"] != null)
                    {
                        linkColumn.ActionParameters = columnNode.Attributes["parameters"].Value;
                    }

                    if (columnNode.Attributes["href"] != null)
                    {
                        linkColumn.NavigateUrl = columnNode.Attributes["href"].Value;
                    }

                    if (columnNode.Attributes["externalsourcename"] != null)
                    {
                        linkColumn.ExternalSourceName = columnNode.Attributes["externalsourcename"].Value;
                        linkColumn.OnExternalDataBound += new OnExternalDataBoundEventHandler(RaiseExternalDataBound);
                    }

                    if (columnNode.Attributes["localize"] != null)
                    {
                        linkColumn.LocalizeStrings = ValidationHelper.GetBoolean(columnNode.Attributes["localize"].Value, false);
                    }

                    if (columnNode.Attributes["style"] != null)
                    {
                        linkColumn.Style = columnNode.Attributes["style"].Value;
                    }
                }
                else
                {
                    BoundField userColumn = new BoundField(); // custom column of the grid view

                    field = (DataControlField)userColumn;

                    // Attribute "source"
                    if (columnNode.Attributes["source"] != null)
                    {
                        columnSource = columnNode.Attributes["source"].Value;
                        userColumn.DataField = columnSource;
                        userColumn.SortExpression = columnSource;
                    }
                }

                field.HeaderStyle.Wrap = false;

                // attribute "caption"
                if (columnNode.Attributes["caption"] != null)
                {
                    columnCaption = columnNode.Attributes["caption"].Value;
                    columnCaption = GridViewTemplate.GetResourceName(columnCaption);
                    field.HeaderText = ResHelper.GetString(columnCaption);
                }

                // attribute "width"
                if (columnNode.Attributes["width"] != null)
                {
                    field.ItemStyle.Width = new Unit(columnNode.Attributes["width"].Value);
                }

                // attribute "wrap"
                if (columnNode.Attributes["wrap"] != null)
                {
                    if (columnNode.Attributes["wrap"].Value.ToLower() == "false")
                    {
                        field.ItemStyle.Wrap = false;
                    }
                }

                // process "filter" node
                if (displayFilter)
                {
                    string filterType = string.Empty;

                    XmlNode filterNode = columnNode.SelectSingleNode("filter");
                    if (filterNode != null)
                    {
                        filterType = filterNode.Attributes["type"].Value;
                        AddFilterField(filterType, columnSource, ResHelper.GetString(columnCaption), filterTable);
                    }
                }

                // Add custom column to gridview
                UniGridView.Columns.Add(field);
            }

            if (displayFilter)
            {
                // add button to the bottom of the filter table
                Button showButton = new Button();
                Literal ltlBreak = new Literal();
                TableRow tRow = new TableRow();
                TableCell tCell = new TableCell();
                showButton.ID = "btnShow";
                showButton.Text = ResHelper.GetString("unigrid.show");
                showButton.SkinID = "ContentButton";
                showButton.Click += new EventHandler(ShowButton_Click);
                ltlBreak.Text = "<br /><br />";
                tCell.Controls.Add(showButton);
                tCell.Controls.Add(ltlBreak);
                tRow.Cells.Add(tCell);
                filterTable.Rows.Add(tRow);
                pnlHeader.DefaultButton = showButton.ID;
            }

            // set pagging acording to the key value "DisplayPageSizeDropdown"                
            displayPageSize = (GetKeyValue(optionNode, "DisplayPageSizeDropdown").ToLower() == "true");

            if (this.DataSource == null)
            {
                // Get query information
                XmlNode queryNode = node.SelectSingleNode("query");
                if (queryNode != null)
                {
                    Query = queryNode.Attributes["name"].Value;

                    // Load the query parameters
                    XmlNodeList parameters = queryNode.SelectNodes("parameter");
                    if ((parameters != null) && (parameters.Count > 0))
                    {
                        object[,] newParams = new object[parameters.Count, 3];
                        int paramIndex = 0;
                        foreach (XmlNode param in parameters)
                        {
                            newParams[paramIndex, 0] = param.Attributes["name"].Value;
                            switch (param.Attributes["type"].Value.ToLower())
                            {
                                case "string":
                                    newParams[paramIndex, 1] = param.Attributes["value"].Value;
                                    break;
                                case "int":
                                    newParams[paramIndex, 1] = ValidationHelper.GetInteger(param.Attributes["value"].Value, 0);
                                    break;
                                case "double":
                                    newParams[paramIndex, 1] = Convert.ToDouble(param.Attributes["value"].Value);
                                    break;
                                case "bool":
                                    newParams[paramIndex, 1] = Convert.ToBoolean(param.Attributes["value"].Value);
                                    break;
                            }
                            this.QueryParameters = newParams;
                        }
                    }
                }
            }

            // Get data from database and set them to the grid view
            ReloadData();

            //filter.Visible = displayFilter;

            //// data source is empty
            //if (isEmpty)
            //{
            //    if (HideControlForZeroRows)
            //    {
            //        // hide filter
            //        filter.Visible = false;
            //    }
            //    else
            //    {
            //        // display zero rows text
            //        lblInfo.Text = ZeroRowsText;

            //        // TODO: display column names
            //    }

            //}
        }

    }


    /// <summary>
    /// Reloads the grid data
    /// </summary>
    public void ReloadData()
    {
        DataSet ds = null;        

        // if datasource for unigrid is query (not dataset), then execute query
        if ((this.Query != null) && (this.Query != string.Empty))
        {
            // Reload the data with current parameters
            GeneralConnection con = ConnectionHelper.GetConnection();
            ds = con.ExecuteQuery(this.Query, this.QueryParameters, this.CompleteWhereCondition, this.OrderBy);
        }
        // external dataset is used
        else
        {
            if (this.OnDataReload != null)
            {
                ds = OnDataReload(CompleteWhereCondition);
            }
            else
            {
                ds = (DataSet)(this.DataSource);
            }            
        }
        
        if (DataHelper.DataSourceIsEmpty(ds))
        {
            ds = new DataSet();
            ds.Tables.Add();         
            UniGridView.DataSource = ds;
        }
        else
        {
            UniGridView.DataSource = ds;
            // Sort data only when not some data exist
            SortUniGridDataSource();
        }
       
        SetUnigridControls();
        UniGridView.DataBind();

    }


    /// <summary>
    /// Returns attribute value of the specific option (specific key node)
    /// </summary>
    /// <param name="optionNode">option node containing key nodes</param>
    /// <param name="keyName">Attribute name of the specific key node</param>
    /// <returns>Attribute value of the specific key node</returns>
    protected string GetKeyValue(XmlNode optionNode, string keyName)
    {
        string keyValue = "";

        if (optionNode != null)
        {
            XmlNode keyNode = optionNode.SelectSingleNode("key[@name='" + keyName + "']");
            if (keyNode != null)
            {
                keyValue = keyNode.Attributes["value"].Value;
            }
        }

        return keyValue;
    }


    /// <summary>
    /// Add filter field to the filter table
    /// </summary>
    /// <param name="type">field type</param>
    /// <param name="column">field id</param>
    /// <param name="column">field display name</param>
    /// <param name="filterTable">filter table name</param>
    protected void AddFilterField(string filedType, string fieldSourceName, string fieldDisplayName, Table filterTable)
    {

        TableRow tRow = new TableRow();

        TableCell tCellName = new TableCell();
        TableCell tCellOption = new TableCell();
        TableCell tCellValue = new TableCell();

        Label textName = new Label();
        textName.Text = fieldDisplayName;
        textName.ID = fieldSourceName + "Name";
        tCellName.Controls.Add(textName);
        tRow.Cells.Add(tCellName);


        switch (filedType)
        {
            case "text":

                DropDownList textOptionFilterField = new DropDownList();
                textOptionFilterField.Items.Add(new ListItem("LIKE", "LIKE"));
                textOptionFilterField.Items.Add(new ListItem("=", "="));
                textOptionFilterField.Items.Add(new ListItem("<>", "<>"));
                textOptionFilterField.ID = fieldSourceName;
                tCellOption.Controls.Add(textOptionFilterField);
                tRow.Cells.Add(tCellOption);

                TextBox textValueFilterField = new TextBox();
                textValueFilterField.ID = fieldSourceName + "TextValue";
                textValueFilterField.Text = "";
                tCellValue.Controls.Add(textValueFilterField);
                tRow.Cells.Add(tCellValue);

                break;

            case "checkbox":

                CheckBox checkboxFilterField = new CheckBox();
                checkboxFilterField.ID = fieldSourceName;
                tCellValue.Controls.Add(checkboxFilterField);
                tRow.Cells.Add(tCellValue);

                break;

            case "integer":
            case "double":

                DropDownList numberOptionFilterField = new DropDownList();
                numberOptionFilterField.Items.Add(new ListItem("=", "="));
                numberOptionFilterField.Items.Add(new ListItem("<>", "<>"));
                numberOptionFilterField.Items.Add(new ListItem("<", "<"));
                numberOptionFilterField.Items.Add(new ListItem(">", ">"));
                numberOptionFilterField.ID = fieldSourceName;
                tCellOption.Controls.Add(numberOptionFilterField);
                tRow.Cells.Add(tCellOption);

                TextBox numberValueFilterField = new TextBox();
                numberValueFilterField.ID = fieldSourceName + "NumberValue";
                numberValueFilterField.Text = "";

                RangeValidator numberValidator = new RangeValidator();
                numberValidator.ID = "numberValidator" + numberValueFilterField.ID;
                numberValidator.Type = (filedType == "integer") ? ValidationDataType.Integer : ValidationDataType.Double;
                numberValidator.MinimumValue = "0";
                numberValidator.MaximumValue = "1000";
                numberValidator.ControlToValidate = numberValueFilterField.ID;
                numberValidator.ErrorMessage = "Number type must be " + filedType + " less than " + numberValidator.MaximumValue + "!";

                tCellValue.Controls.Add(numberValueFilterField);
                tCellValue.Controls.Add(numberValidator);
                tRow.Cells.Add(tCellValue);

                break;

        }

        filterTable.Rows.Add(tRow);
    }


    /// <summary>
    /// Set filter to the grid view and save it to the view state
    /// </summary>    
    protected void SetFilter()
    {

        string where = string.Empty;
        int whereConditionCount = 0;    // count of the conditions in the 'where clause'
        string andExpression;           // AND in 'where clause'                        

        // go through the table of filter controls
        //--------------------------------------------

        // go through all rows, except the last one with submit button
        for (int i = 0; i < filter.Controls[0].Controls.Count - 1; i++)
        {
            // go through all cells in current row
            for (int j = 0; j < filter.Controls[0].Controls[i].Controls.Count; j++)
            {
                // get controls from each row cell
                foreach (Control filterControl in filter.Controls[0].Controls[i].Controls[j].Controls)
                {
                    string controlType;
                    controlType = filter.Controls[0].Controls[i].Controls[j].FindControl(filterControl.ID).GetType().ToString().ToLower();
                    controlType = controlType.Substring(controlType.LastIndexOf('.') + 1);

                    switch (controlType)
                    {
                        case "dropdownlist":

                            DropDownList ddlistControl = (DropDownList)filterControl;
                            string textboxValue = ((TextBox)filter.Controls[0].Controls[i].Controls[j + 1].Controls[0]).Text.Replace("\'","''");   // value from the textbox
                            string textboxID = ((TextBox)filter.Controls[0].Controls[i].Controls[j + 1].Controls[0]).ID;        // ID of the textbox

                            // empty field -> no filter is set for this field
                            if (textboxValue != "")
                            {
                                string textboxSqlValue;  // value used in 'where clause' of the sql command

                                if (textboxID.EndsWith("TextValue"))
                                {

                                    if (ddlistControl.SelectedValue == "LIKE")
                                    { // option: LIKE

                                        textboxSqlValue = "N'%" + textboxValue + "%'";
                                    }
                                    else // options: =, <>
                                        textboxSqlValue = "N'" + textboxValue + "'";

                                }
                                else // textboxID.EndsWith("NumberValue")
                                {
                                    textboxSqlValue = textboxValue;
                                }

                                andExpression = (whereConditionCount > 0) ? " AND" : "";

                                // ddlistControl.ID                - column name
                                // ddlistControl.SelectedValue     - condition option
                                // textboxSqlValue                  - condition value                        
                                where += andExpression + " " + ddlistControl.ID + " " + ddlistControl.SelectedValue + " " + textboxSqlValue;
                                whereConditionCount++;
                            }
                            break;


                        case "checkbox":

                            CheckBox currentControl = (CheckBox)filterControl;
                            if (currentControl.Checked)
                            {
                                andExpression = (whereConditionCount > 0) ? " AND" : "";

                                where += andExpression + " " + currentControl.ID + " = 1";
                                whereConditionCount++;
                            }
                            break;

                        default:
                            // do nothing
                            break;

                    }
                }
            }
        }
        WhereClause = where;

        // Get data from database and set them to the grid view
        ReloadData();
    }


    /// <summary>
    /// Process data from filter
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void ShowButton_Click(object sender, EventArgs e)
    {
        SetFilter();
    }


    /// <summary>
    /// Raises the external databound event
    /// </summary>
    /// <param name="sourceName">External source name</param>
    /// <param name="parameter">Source parameter</param>
    private object RaiseExternalDataBound(object sender, string sourceName, object parameter)
    {
        return OnExternalDataBound(sender, sourceName, parameter);
    }


    protected void UniGridView_PageIndexChanging(object sender, GridViewPageEventArgs e)
    {
        // Set page index
        UniGridView.PageIndex = e.NewPageIndex;

        SortUniGridDataSource();
        UniGridView.DataBind();

    }


    protected void UniGridView_Sorting(object sender, EventArgs e)
    {
        // sorting is handled on RowCommand event 
    }


    protected void UniGridView_RowCommand(object sender, GridViewCommandEventArgs e)
    {
        // if sorting by column is called
        if (e.CommandName.ToLower() == "sort")
        {

            string orderByColumn = Convert.ToString(e.CommandArgument);

            // if sorting is called for the first time and default sorting (OrderBy property) is set
            if ((SortDirect == "") && (this.OrderBy != null) && (this.OrderBy != ""))
            {
                ChangeSortDirection(orderByColumn, this.OrderBy);
            }
            else
            {
                ChangeSortDirection(orderByColumn, this.SortDirect);
            }

            SortUniGridDataSource();
            UniGridView.DataBind();
        }

        // other cases (button clicks) except pagging
        else if (e.CommandName.ToLower() != "page")
        {
            if (OnAction != null)
            {
                OnAction(e.CommandName, e.CommandArgument);
            }

            //// get query information
            //XmlNode queryNode = node.SelectSingleNode("query");
            //Query = queryNode.InnerText;

            // Get data from database and set them to the grid view
            ReloadData();
        }

    }


    /// <summary>
    /// Sorts UniGrid data source according to sort directive saved in viewstate
    /// </summary>
    protected void SortUniGridDataSource()
    {
        if (SortDirect != "")
        {
            // Set sort directive from viewstate
            if (UniGridView.DataSource is DataTable)
            {
                ((DataTable)(UniGridView.DataSource)).DefaultView.Sort = SortDirect;
            }
            else if (UniGridView.DataSource is DataSet)
            {
                ((DataSet)(UniGridView.DataSource)).Tables[0].DefaultView.Sort = SortDirect;
                UniGridView.DataSource = ((DataSet)(UniGridView.DataSource)).Tables[0];
            }
        }
    }


    /// <summary>
    /// Changes sorting direction by specified column
    /// </summary>
    /// <param name="orderByColumn">Column name to order by.</param>
    /// <param name="orderByString">Old order by string.</param> 
    protected void ChangeSortDirection(string orderByColumn, string orderByString)
    {
        // sort by the same column -> the other directon
        if (orderByString.ToLower().Trim().StartsWith(orderByColumn.ToLower()))
        {
            SortDirect = (orderByString.ToLower().Trim().EndsWith("desc")) ? orderByColumn + " asc" : orderByColumn + " desc";
        }
        // sort by a new column -> implicitly direction is ASC
        else
        {
            SortDirect = orderByColumn + " asc";
        }
    }


    /// <summary>
    /// Sets unigrid controls.
    /// </summary>
    protected void SetUnigridControls()
    {
        pnlFooter.Visible = false;
        filter.Visible = displayFilter;

        // Indicates whether unigrid datasource is empty or not
        isEmpty = DataHelper.DataSourceIsEmpty(UniGridView.DataSource);

        if (HideControlForZeroRows && isEmpty)
        {            
            // hide filter
            filter.Visible = false;
        }
        else
        {
            if (displayPageSize)
            {
                // display pagging dropdown
                pnlFooter.Visible = true;

                if (pageSizeDropdown.SelectedValue.ToLower() == "all")
                {
                    UniGridView.AllowPaging = false;
                }
                else
                {
                    UniGridView.AllowPaging = true;
                    UniGridView.PageSize = Convert.ToInt32(pageSizeDropdown.SelectedValue);
                }
            }
        }
        
        // data source is empty
        if (isEmpty && !HideControlForZeroRows)
        {
                // display zero rows text
                lblInfo.Text = ZeroRowsText;
        }
    }


    protected void UniGridView_RowDeleting(object sender, GridViewDeleteEventArgs e)
    {
        // deleting is handled in custom method
    }

    protected void UniGridView_RowEditing(object sender, GridViewEditEventArgs e)
    {
        // editing is handled in custom method
    }

    protected void UniGridView_RowUpdating(object sender, GridViewUpdateEventArgs e)
    {
        // updating is handled in custom method
    }


}
