using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using CMS.GlobalHelper;
using CMS.CMSHelper;
using CMS.Blogs;
using CMS.SiteProvider;

public partial class CMSBlog_BlogCommentEdit : System.Web.UI.UserControl
{
    private int mPostDocumentId = 0;
    private int mCommentId = 0;
    private UserInfo currentUser = CMSContext.CurrentUser;
    private string mOkButtonText = ResHelper.GetString("Blog.CommentEdit.btnOk");
    private string mCommentSavedText = ResHelper.GetString("General.ChangesSaved");
    private bool mClearFormAfterSave = false;
    private bool mUseCaptcha = false;

    public event OnBeforeCommentSavedEventHandler OnBeforeCommentSaved;
    public event OnAfterCommentSavedEventHandler OnAfterCommentSaved;



    #region " Public properties "

    /// <summary>
    /// Post document ID. Set when creating new comment to the post.
    /// </summary>
    public int PostDocumentId
    {
        get
        {
            return mPostDocumentId;
        }
        set
        {
            mPostDocumentId = value;
        }
    }


    /// <summary>
    /// Comment ID. Set when editing existing comment.
    /// </summary>
    public int CommentId
    {
        get
        {
            return mCommentId;
        }
        set
        {
            mCommentId = value;
        }
    }


    /// <summary>
    /// Ok button text.
    /// </summary>
    public string OkButtonText
    {
        get
        {
            return mOkButtonText;
        }
        set
        {
            mOkButtonText = value;
        }
    }


    /// <summary>
    /// Comment saved text.
    /// </summary>
    public string CommentSavedText
    {
        get
        {
            return mCommentSavedText;
        }
        set
        {
            mCommentSavedText = value;
        }
    }


    /// <summary>
    /// Indicates whether form fields should be cleared after data are saved.
    /// </summary>
    public bool ClearFormAfterSave
    {
        get
        {
            return mClearFormAfterSave;
        }
        set
        {
            mClearFormAfterSave = value;
        }
    }


    /// <summary>
    /// Indicates whether security code control should be displayed.
    /// </summary>
    public bool UseCaptcha
    {
        get
        {
            return mUseCaptcha;
        }
        set
        {
            mUseCaptcha = value;
        }
    }

    #endregion


    #region " Private properties"

    private bool IsInsertMode
    {
        get
        {
            // Insert mode
            if (mPostDocumentId > 0)
            {
                return true;
            }
            // Edit mode
            else
            {
                return false;
            }
        }
    }

    #endregion


    protected void Page_Load(object sender, EventArgs e)
    {
        // Basic control initialization
        lblName.Text = ResHelper.GetString("Blog.CommentEdit.lblName");
        lblUrl.Text = ResHelper.GetString("Blog.CommentEdit.lblUrl");
        lblComments.Text = ResHelper.GetString("Blog.CommentEdit.lblComments");
        btnOk.Text = mOkButtonText;

        // Validators initialization
        rfvComments.ErrorMessage = ResHelper.GetString("Blog.CommentEdit.CommentEmpty");
        rfvName.ErrorMessage = ResHelper.GetString("Blog.CommentEdit.NameEmpty");

        // Show or hide security code control
        plcCaptcha.Visible = mUseCaptcha;
        lblCaptcha.Text = ResHelper.GetString("SecurityCode.lblSecurityCode");             

        // New comment
        if (IsInsertMode)
        {
            // Prefill current user fullname
            if ((currentUser != null) && (!currentUser.IsPublic()))
            {
                if (currentUser.UserNickName != "")
                {
                    txtName.Text = currentUser.UserNickName;
                }
                else
                {
                    txtName.Text = currentUser.FullName;
                }
                
            }
        }
        // Existing comment
        else if (!IsPostBack)
        {
            LoadCommentData();
        }

    }

    /// <summary>
    /// Fill form with the comment data.
    /// </summary>
    protected void LoadCommentData()
    {
        // Get comment info from database
        BlogCommentInfo bci = BlogCommentInfoProvider.GetBlogCommentInfo(mCommentId);
        if (bci != null)
        {
            txtName.Text = bci.CommentUserName;
            txtUrl.Text = bci.CommentUrl;
            txtComments.Text = bci.CommentText;
        }
    }


    protected void btnOk_Click(object sender, EventArgs e)
    {
        if (OnBeforeCommentSaved != null)
        {
            OnBeforeCommentSaved();
        }

        BlogCommentInfo bci = null;
        string errorMessage = "";  

        // Validate form
        errorMessage = ValidateForm();
        if (errorMessage == "")
        {
            // Create new comment
            if (IsInsertMode)
            {
                bci = new BlogCommentInfo();
                bci.CommentDate = DateTime.Now;
                bci.CommentPostDocumentID = mPostDocumentId;
                bci.CommentUserID = (currentUser != null) ? currentUser.UserID : 0;
            }
            // Get existing comment
            else
            {
                bci = BlogCommentInfoProvider.GetBlogCommentInfo(mCommentId);
            }

            // Update basic comment properties
            if (bci != null)
            {
                bci.CommentUserName = HTMLHelper.StripTags(txtName.Text.Trim());
                bci.CommentUrl = HTMLHelper.StripTags(txtUrl.Text.Trim());
                bci.CommentText = HTMLHelper.StripTags(txtComments.Text.Trim());
            }

            try
            {
                // Save changes to database
                BlogCommentInfoProvider.SetBlogCommentInfo(bci);
                lblInfo.Visible = true;
                lblInfo.Text = this.CommentSavedText;

                // Clear form when required
                if (this.mClearFormAfterSave)
                {
                    txtComments.Text = "";
                    txtUrl.Text = "";
                    ctrlCaptcha.Value = "";
                }

                if (OnAfterCommentSaved != null)
                {
                    OnAfterCommentSaved(bci);
                }
            }
            catch (Exception ex)
            {
                errorMessage = ex.Message;
            }

        }

        if (errorMessage != "")
        {
            // Show error message
            lblError.Visible = true;
            lblError.Text = errorMessage;
        }
        else
        {
            // Regenerate captcha code when comment saved successfully
            this.ctrlCaptcha.GenerateNewCode();
        }
    }


    /// <summary>
    /// Validates form.
    /// </summary>
    /// <returns></returns>
    protected string ValidateForm()
    {
        string errorMessage = "";

        errorMessage = new Validator().NotEmpty(txtComments.Text.Trim(), rfvComments.ErrorMessage).NotEmpty(txtName.Text.Trim(), rfvName.ErrorMessage).Result;

        if ((mUseCaptcha)&& (errorMessage == ""))
        {
            // Check whether security code is correct
            if (!ctrlCaptcha.IsValid())
            {
                errorMessage = ctrlCaptcha.ValidationError;
            }
        }

        return errorMessage;
    }
}
