using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using CMS.GlobalHelper;
using CMS.CMSHelper;
using CMS.Blogs;
using CMS.SiteProvider;
using CMS.EmailEngine;
using CMS.EventLog;
using CMS.TreeEngine;
using CMS.SettingsProvider;
using CMS.WorkflowEngine;

public partial class CMSBlog_BlogCommentView : System.Web.UI.UserControl
{
    private BlogProperties mBlogProperties = new BlogProperties();
    private string mCommentDetailControlPath = "~/CMSBlog/BlogCommentDetail.ascx";

    private bool isUserAuthorized = false;
    private CurrentUserInfo currrentUser = CMSContext.CurrentUser;

    private CMS.TreeEngine.TreeNode mPostNode = null;

    private string mAliasPath = null;
    private string mCulture = null;
    private string mSiteName = null;


    #region "Public properties"

    /// <summary>
    /// Post alias path
    /// </summary>
    public string AliasPath
    {
        get
        {
            if (mAliasPath == null)
            {
                mAliasPath = CMSContext.CurrentPageInfo.NodeAliasPath;
            }
            return mAliasPath;
        }
        set
        {
            mAliasPath = value;
        }
    }


    /// <summary>
    /// Post culture
    /// </summary>
    public string Culture
    {
        get
        {
            if (mCulture == null)
            {
                mCulture = CMSContext.CurrentUser.PreferredCultureCode;
            }
            return mCulture;
        }
        set
        {
            mCulture = value;
        }
    }


    /// <summary>
    /// Post SiteName
    /// </summary>
    public string SiteName
    {
        get
        {
            if (mSiteName == null)
            {
                mSiteName = CMSContext.CurrentSiteName;
            }
            return mSiteName;
        }
        set
        {
            mSiteName = value;
        }
    }


    /// <summary>
    /// Post document node
    /// </summary>
    public CMS.TreeEngine.TreeNode PostNode
    {
        get
        {
            if (mPostNode == null)
            {
                // Get the document
                TreeProvider tree = new TreeProvider();
                mPostNode = tree.SelectSingleNode(this.SiteName, this.AliasPath, this.Culture, false, "cms.blogpost", true);
                if ((mPostNode != null) && (CMSContext.CurrentUser.ViewMode != ViewModeEnum.LiveSite))
                {
                    mPostNode = DocumentHelper.GetDocument(mPostNode, tree);
                }
            }
            return mPostNode;
        }
        set
        {
            mPostNode = value;
        }
    }


    /// <summary>
    /// Blog properties.
    /// </summary>
    public BlogProperties BlogProperties
    {
        get
        {
            return mBlogProperties;
        }
        set
        {
            mBlogProperties = value;
        }
    }


    /// <summary>
    /// Comment detail control path.
    /// </summary>
    public string CommentDetailControlPath
    {
        get
        {
            return mCommentDetailControlPath;
        }
        set
        {
            mCommentDetailControlPath = value;
        }
    }


    /// <summary>
    /// Indicates whether post comments can be added to the post.
    /// </summary>
    public bool AreCommentsOpened
    {
        get
        {
            bool isOpened = false;

            // Get current post document info            
            if (this.PostNode != null)
            {
                if (!ValidationHelper.GetBoolean(this.PostNode.GetValue("BlogPostAllowComments"), false))
                {
                    // Comments are not allowed for current post
                    isOpened = false;
                }
                else
                {
                    // Check new comment dialog expiration
                    switch (this.BlogProperties.OpenCommentsFor)
                    {
                        case BlogProperties.OPEN_COMMENTS_ALWAYS:
                            isOpened = true;
                            break;

                        case BlogProperties.OPEN_COMMENTS_DISABLE:
                            isOpened = false;
                            break;

                        default:
                            DateTime postDate = ValidationHelper.GetDateTime(this.PostNode.GetValue("BlogPostDate"), DataHelper.DATETIME_NOT_SELECTED);
                            isOpened = (DateTime.Now <= postDate.AddDays(this.BlogProperties.OpenCommentsFor));

                            break;
                    }

                    if (currrentUser.IsPublic())
                    {
                        isOpened = (isOpened && this.BlogProperties.AllowAnonymousComments);
                    }
                }
            }

            return isOpened;
        }
    }


    #endregion


    protected void Page_Load(object sender, EventArgs e)
    {
        // Check permissions for blog
        if (this.BlogProperties.CheckPermissions)
        {
            if (CMSContext.CurrentUser.IsAuthorizedPerDocument(this.PostNode, NodePermissionsEnum.Read) != AuthorizationResultEnum.Allowed)
            {
                this.Visible = false;
                return;
            }
        }

        lblTitle.Text = ResHelper.GetString("Blog.CommentView.Comments");
        lblLeaveComment.Text = ResHelper.GetString("Blog.CommentView.LeaveComment");

        if (this.PostNode != null)
        {
            rptComments.ItemDataBound += new RepeaterItemEventHandler(rptComments_ItemDataBound);
            ctrlCommentEdit.OnAfterCommentSaved += new OnAfterCommentSavedEventHandler(ctrlCommentEdit_OnAfterCommentSaved);
            ctrlCommentEdit.UseCaptcha = this.BlogProperties.UseCaptcha;
            ctrlCommentEdit.ClearFormAfterSave = true;

            if (this.BlogProperties.ModerateComments)
            {
                ctrlCommentEdit.CommentSavedText = ResHelper.GetString("Blog.CommentView.ModeratedCommentSaved");
            }
            else
            {
                ctrlCommentEdit.CommentSavedText = ResHelper.GetString("Blog.CommentView.CommentSaved");
            }
            
        }

        ReloadData();
    }


    /// <summary>
    /// Reloads comment list after new comment is added
    /// </summary>    
    void ctrlCommentEdit_OnAfterCommentSaved(BlogCommentInfo commentObj)
    {
        ReloadData();

        // Send comment notification to specified email address
        if (this.BlogProperties.SendCommentsToEmail != "")
        {
            SendNewCommentNotification(commentObj);
        }
    }


    void mBlogComment_OnCommentAction(string actionName, object actionArgument)
    {
        // Get comment ID
        int commentId = ValidationHelper.GetInteger(actionArgument, 0);
        BlogCommentInfo bci = null;

        switch (actionName.ToLower())
        {

            case "delete":
                // Check 'Manage' permission
                if (!isUserAuthorized)
                {
                    CMSDeskPage.RedirectToAccessDenied("cms.blog", "Manage");
                }

                // Delete comment
                BlogCommentInfoProvider.DeleteBlogCommentInfo(commentId);

                ReloadData();
                break;

            case "approve":
                // Check 'Manage' permission
                if (!isUserAuthorized)
                {
                    CMSDeskPage.RedirectToAccessDenied("cms.blog", "Manage");
                }

                // Set comment as 'approved'
                bci = BlogCommentInfoProvider.GetBlogCommentInfo(commentId);
                if ((bci != null) && (currrentUser != null))
                {
                    bci.CommentApprovedByUserID = currrentUser.UserID;
                    BlogCommentInfoProvider.SetBlogCommentInfo(bci);
                }

                ReloadData();
                break;

            case "reject":
                // Check 'Manage' permission
                if (!isUserAuthorized)
                {
                    CMSDeskPage.RedirectToAccessDenied("cms.blog", "Manage");
                }

                // Set comment as 'rejected'
                bci = BlogCommentInfoProvider.GetBlogCommentInfo(commentId);
                if (bci != null)
                {
                    bci.CommentApprovedByUserID = 0;
                    BlogCommentInfoProvider.SetBlogCommentInfo(bci);
                }

                ReloadData();
                break;
        }

    }


    /// <summary>
    /// Reloads comment list.
    /// </summary>
    protected void ReloadData()
    {

        // Check permissions for blog
        if (this.BlogProperties.CheckPermissions)
        {
            if (CMSContext.CurrentUser.IsAuthorizedPerDocument(this.PostNode, NodePermissionsEnum.Read) != AuthorizationResultEnum.Allowed)
            {
                this.Visible = false;
                return;
            }
        }

        if (this.PostNode != null)
        {
            ctrlCommentEdit.PostDocumentId = this.PostNode.DocumentID;
            ctrlCommentEdit.Visible = this.AreCommentsOpened;
            lblLeaveComment.Visible = this.AreCommentsOpened;

            if (!this.BlogProperties.StopProcessing)
            {
                bool isOwner = false;
                bool isModerated = false;
                bool onlyApproved = false;

                // Is user authorized to manage comments?
                CMS.TreeEngine.TreeNode blogNode = BlogHelper.GetParentBlog(this.AliasPath, this.SiteName);
                if (blogNode != null)
                {
                    isOwner = (currrentUser.UserID == ValidationHelper.GetInteger(blogNode.GetValue("NodeOwner"), 0));
                    isModerated = ValidationHelper.GetBoolean(blogNode.GetValue("BlogModerateComments"), false);
                }
                isUserAuthorized = (currrentUser.IsAuthorizedPerResource("cms.blog", "Manage") || isOwner);
                onlyApproved = (isModerated && !isUserAuthorized);

                // Get all post comments
                rptComments.DataSource = BlogCommentInfoProvider.GetPostComments(this.PostNode.DocumentID, onlyApproved);
                rptComments.DataBind();

                if (DataHelper.DataSourceIsEmpty(rptComments.DataSource))
                {
                    lblNoComments.Visible = true;
                    lblNoComments.Text = ResHelper.GetString("CommentView.NoComments");
                }
            }
        }
    }

    /// <summary>
    /// Adds comment list item.
    /// </summary>
    void rptComments_ItemDataBound(object sender, RepeaterItemEventArgs e)
    {
        if (e.Item.DataItem != null)
        {
            // Create new comment info object
            BlogCommentInfo bci = new BlogCommentInfo(((DataRowView)e.Item.DataItem).Row);

            // Load 'BlogCommentDetail.ascx' control
            BlogCommentDetail mBlogComment = (BlogCommentDetail)this.LoadControl(mCommentDetailControlPath);

            // Set control data and properties
            mBlogComment.BlogpPoperties = mBlogProperties;
            mBlogComment.CommentsDataRow = ((DataRowView)e.Item.DataItem).Row;

            // Initialiaze comment 'Action buttons'
            mBlogComment.OnCommentAction += new OnCommentActionEventHandler(mBlogComment_OnCommentAction);
            mBlogComment.ShowApproveButton = ((mBlogComment.BlogpPoperties.ModerateComments) && isUserAuthorized && (bci.CommentApprovedByUserID == 0));
            mBlogComment.ShowRejectButton = ((mBlogComment.BlogpPoperties.ModerateComments) && isUserAuthorized && (bci.CommentApprovedByUserID != 0));
            mBlogComment.ShowDeleteButton = (mBlogProperties.ShowDeleteButton && isUserAuthorized);
            mBlogComment.ShowEditButton = (mBlogProperties.ShowEditButton && isUserAuthorized);

            // Add loaded control as comment list item
            e.Item.Controls.Clear();
            e.Item.Controls.Add(mBlogComment);
            e.Item.Controls.Add(new LiteralControl("<hr class=\"PostCommentSeparatorLine\" >"));
        }
    }


    /// <summary>
    /// Sends order notification to customer.
    /// </summary>
    /// <param name="bciObj">Blog comment info object.</param>           
    private void SendNewCommentNotification(BlogCommentInfo bciObj)
    {
        SiteInfo currentSite = CMSContext.CurrentSite;        

        if ((currentSite != null) && (bciObj != null))
        {
            EmailTemplateInfo emailTemplate = EmailTemplateProvider.GetEmailTemplate("Blog.NewCommentNotification", currentSite.SiteName);
            SiteInfo site = SiteInfoProvider.GetSiteInfo(currentSite.SiteName);

            if ((emailTemplate != null) && (site != null))
            {
                if (this.BlogProperties.SendCommentsToEmail != "")
                {
                    // Get post name
                    string postName = "";
                    if (this.PostNode != null)
                    {
                        postName = ValidationHelper.GetString(PostNode.GetValue("BlogPostTitle"), "");
                    }                    

                    // Initialize email message
                    EmailMessage message = new EmailMessage();                    
                    message.From = SettingsKeyProvider.GetStringValue(CMSContext.CurrentSiteName + ".CMSSendBlogEmailsFrom");
                    message.Subject = ResHelper.GetString("blog.newcommentnotification.emailsubject");
                    message.Recipients = this.BlogProperties.SendCommentsToEmail;
                    message.EmailFormat = EmailFormatEnum.Html;

                    // Macros
                    string[,] macros = new string[5, 2];
                    macros[0, 0] = "USERFULLNAME";
                    macros[0, 1] = bciObj.CommentUserName;
                    macros[1, 0] = "COMMENTURL";
                    macros[1, 1] = bciObj.CommentUrl;
                    macros[2, 0] = "COMMENTS";
                    macros[2, 1] = bciObj.CommentText;
                    macros[3, 0] = "COMMENTDATE";
                    macros[3, 1] = bciObj.CommentDate.ToString();
                    macros[4, 0] = "BLOGPOSTTITLE";
                    macros[4, 1] = postName;

                    try
                    {
                        // Send email
                        EmailSender.SendEmailWithTemplateText(currentSite.SiteName, message, emailTemplate.TemplateText, macros);
                    }
                    catch (Exception ex)
                    {
                        try
                        {
                            EventLogProvider eventLog = new EventLogProvider();
                            eventLog.LogEvent("Send new comment notification email", "BLOG", ex);
                        }
                        catch
                        {
                            // Unable to log the event
                        }
                    }
                }
            }
        }
    }
}
