using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using CMS.PortalControls;
using CMS.Polls;
using CMS.CMSHelper;
using CMS.GlobalHelper;
using CMS.ExtendedControls;
using CMS.SiteProvider;

public partial class CMSWebParts_Polls_PollView : System.Web.UI.UserControl
{
    protected string mPollCodeName = null;
    protected bool mShowGraph = true;
    protected CountTypeEnum mCodeType = CountTypeEnum.Absolute;
    protected bool mShowResultsAfterVote = true;
    protected bool mCheckVoted = true;
    protected bool mCheckPermissions = true;
    protected bool mCheckOpen = true;
    protected bool mHideWhenNotAuthorized = false;
    protected bool mHideWhenNotOpened = false;
    protected string mButtonText = null;


    #region "Properties"

    /// <summary>
    /// Code name of the poll, which should be displayed.
    /// </summary>
    public string PollCodeName
    {
        get
        {
            return mPollCodeName;
        }
        set
        {
            mPollCodeName = value;
        }
    }


    /// <summary>
    /// If true, graph of the poll is displayed.
    /// </summary>
    public bool ShowGraph
    {
        get
        {
            return mShowGraph;
        }
        set
        {
            mShowGraph = value;
        }
    }


    /// <summary>
    /// Type of the representation of the answers count in the graph.
    /// </summary>
    public CountTypeEnum CountType
    {
        get
        {
            return mCodeType;
        }
        set
        {
            mCodeType = value;
        }
    }


    /// <summary>
    /// If true, graph is displayed after answering the poll.
    /// </summary>
    public bool ShowResultsAfterVote
    {
        get
        {
            return mShowResultsAfterVote;
        }
        set
        {
            mShowResultsAfterVote = value;
        }
    }


    /// <summary>
    /// Check whether current user has voted.
    /// </summary>
    public bool CheckVoted
    {
        get
        {
            return mCheckVoted;
        }
        set
        {
            mCheckVoted = value;
        }
    }


    /// <summary>
    /// Check permissions.
    /// </summary>
    public bool CheckPermissions
    {
        get
        {
            return mCheckPermissions;
        }
        set
        {
            mCheckPermissions = value;
        }
    }


    /// <summary>
    /// Check open from/to time.
    /// </summary>
    public bool CheckOpen
    {
        get
        {
            return mCheckOpen;
        }
        set
        {
            mCheckOpen = value;
        }
    }


    /// <summary>
    /// If true, the control hides when not authorized, 
    /// otherwise the control displays the message and does not allow to vote.
    /// </summary>
    public bool HideWhenNotAuthorized
    {
        get
        {
            return mHideWhenNotAuthorized;
        }
        set
        {
            mHideWhenNotAuthorized = value;
        }
    }


    /// <summary>
    /// If true, the control hides when not opened, 
    /// otherwise the control does not allow to vote.
    /// </summary>
    public bool HideWhenNotOpened
    {
        get
        {
            return mHideWhenNotOpened;
        }
        set
        {
            mHideWhenNotOpened = value;
        }
    }


    /// <summary>
    /// Text of the vote button.
    /// </summary>
    public string ButtonText
    {
        get
        {
            return ValidationHelper.GetString(mButtonText, ResHelper.GetString("Polls.Vote"));
        }
        set
        {
            mButtonText = value;
        }
    }


    #endregion


    /// <summary>
    /// Poll info.
    /// </summary>
    private PollInfo pi = null;

    /// <summary>
    /// Error message.
    /// </summary>
    protected string errMessage = null;

    /// <summary>
    /// Info about permission.
    /// </summary>
    protected bool hasPermission = false;

    /// <summary>
    /// Info whether the poll is opened.
    /// </summary>
    protected bool isOpened = false;


    protected void Page_Load(object sender, EventArgs e)
    {
        pi = PollInfoProvider.GetPollInfo(this.PollCodeName);
        hasPermission = HasPermission();
        isOpened = IsOpened();

        if ((pi != null) && (hasPermission || !this.HideWhenNotAuthorized) && (isOpened || !this.HideWhenNotOpened) && (pi.IsInSite(CMSContext.CurrentSite.SiteName)))
        {
            // Reload data
            ReloadData(false);
        }
        else
        {
            this.Visible = false;
        }
    }


    /// <summary>
    /// Loads data.
    /// </summary>
    public void ReloadData(bool forceReload)
    {
        if (pi != null)
        {
            // Load title
            lblTitle.Text = pi.PollTitle;
            // Load question
            lblQuestion.Text = pi.PollQuestion;

            if ((!forceReload) || ((forceReload) && (this.ShowResultsAfterVote)))
            {
                // Add answer section
                CreateAnswerSection(forceReload);
            }
            else
            {
                // Hide answer panel
                pnlAnswer.Visible = false;
            }

            if ((forceReload) && (isOpened))
            {
                // Hide vote button
                btnVote.Visible = false;

                // Add poll response after voting
                if ((errMessage != null) && (errMessage.Trim() != ""))
                {
                    // Display message if error occurs
                    lblInfo.Text = errMessage;
                    lblInfo.CssClass = "ErrorMessage";
                }
                else
                {
                    // Display poll response message
                    lblInfo.Text = pi.PollResponseMessage;
                }
            }
            else if (isOpened)
            {
                if (hasPermission && !((this.CheckVoted) && (PollInfoProvider.HasVoted(pi.PollID))))
                {
                    // Add vote button
                    btnVote.Visible = true;
                    btnVote.Text = this.ButtonText;
                }
                else
                {
                    btnVote.Visible = false;
                }
            }
            else
            {
                btnVote.Visible = false;
                lblInfo.Text = ResHelper.GetString("Polls.Closed");
            }
        }
    }


    /// <summary>
    /// Create poll answer section.
    /// </summary>
    /// <param name="reload">Indicates postback.</param>
    protected void CreateAnswerSection(bool reload)
    {
        if (reload)
        {
            pnlAnswer.Controls.Clear();
        }

        if (pi != null)
        {
            // Get poll's answers
            DataSet ds = PollAnswerInfoProvider.GetAnswers(pi.PollID);
            if (!DataHelper.DataSourceIsEmpty(ds))
            {
                int maxCount = 0;
                int sumCount = 0;

                // Sum answer counts and get highest
                foreach (DataRow row in ds.Tables[0].Rows)
                {
                    bool enabled = ValidationHelper.GetBoolean(row["AnswerEnabled"], true);
                    if (enabled)
                    {
                        int count = ValidationHelper.GetInteger(row["AnswerCount"], 0);
                        sumCount += count;
                        if (count > maxCount)
                        {
                            maxCount = count;
                        }
                    }
                }

                LocalizedCheckBox chkItem = null;
                LocalizedRadioButton radItem = null;
                LocalizedLabel lblItem = null;
                int index = 0;

                pnlAnswer.Controls.Add(new LiteralControl("<table cellpadding=\"0\" cellspacing=\"0\" width=\"100%\">"));

                // Create the answers
                foreach (DataRow row in ds.Tables[0].Rows)
                {
                    bool enabled = ValidationHelper.GetBoolean(row["AnswerEnabled"], true);
                    if (enabled)
                    {
                        pnlAnswer.Controls.Add(new LiteralControl("<tr><td class=\"PollAnswer\" colspan=\"2\">"));

                        if (((reload) && (this.ShowResultsAfterVote)) || (!hasPermission && !this.HideWhenNotAuthorized)
                            || (!isOpened && !this.HideWhenNotOpened) || (this.CheckVoted && PollInfoProvider.HasVoted(pi.PollID)))
                        {
                            // Add label
                            lblItem = new LocalizedLabel();
                            lblItem.ID = "lbl" + ValidationHelper.GetInteger(row["AnswerID"], 0);
                            lblItem.EnableViewState = false;
                            lblItem.Text = ValidationHelper.GetString(row["AnswerText"], "");
                            lblItem.CssClass = "PollAnswerText";

                            pnlAnswer.Controls.Add(lblItem);
                        }
                        else
                        {
                            if (pi.PollAllowMultipleAnswers)
                            {
                                // Add checkboxes for multiple answers
                                chkItem = new LocalizedCheckBox();
                                chkItem.ID = "chk" + ValidationHelper.GetInteger(row["AnswerID"], 0);
                                chkItem.AutoPostBack = false;
                                chkItem.Text = ValidationHelper.GetString(row["AnswerText"], "");
                                chkItem.Checked = false;
                                chkItem.CssClass = "PollAnswerCheck";

                                pnlAnswer.Controls.Add(chkItem);
                            }
                            else
                            {
                                // Add radiobuttons
                                radItem = new LocalizedRadioButton();
                                radItem.ID = "rad" + ValidationHelper.GetInteger(row["AnswerID"], 0);
                                radItem.AutoPostBack = false;
                                radItem.GroupName = pi.PollCodeName + "Group";
                                radItem.Text = ValidationHelper.GetString(row["AnswerText"], "");
                                radItem.Checked = false;
                                radItem.CssClass = "PollAnswerRadio";

                                pnlAnswer.Controls.Add(radItem);
                            }
                        }

                        if ((this.ShowGraph) || ((reload) && (this.ShowResultsAfterVote)))
                        {
                            // Create graph under the answer
                            CreateGraph(maxCount, ValidationHelper.GetInteger(row["AnswerCount"], 0), sumCount, index);
                        }

                        pnlAnswer.Controls.Add(new LiteralControl("</td></tr>"));

                        index++;
                    }
                }

                pnlAnswer.Controls.Add(new LiteralControl("</table>"));
            }
        }
    }


    /// <summary>
    /// Create graph bar for the answer.
    /// </summary>
    /// <param name="maxValue">Max answers' count.</param>
    /// <param name="currentValue">Current answer count.</param>
    /// <param name="countSummary">Count summary of all answers.</param>
    /// <param name="index">Index.</param>
    protected void CreateGraph(int maxValue, int currentValue, int countSummary, int index)
    {
        int ratio = 0;
        if (maxValue != 0)
        {
            ratio = (int)Math.Round(((double)currentValue / maxValue) * 100);
        }
        // Begin PollAnswerGraph
        pnlAnswer.Controls.Add(new LiteralControl("<tr><td style=\"width: 100%;\"><div class=\"PollGraph\">"));
        if (ratio != 0)
        {
            // PollAnswerItemGraph
            pnlAnswer.Controls.Add(new LiteralControl("<div class=\"PollGraph" + index +
                "\" style=\"width:" + ratio + "%\">&nbsp;</div>"));
        }
        else
        {
            pnlAnswer.Controls.Add(new LiteralControl("&nbsp;"));
        }

        // End PollAnswerGraph
        pnlAnswer.Controls.Add(new LiteralControl("</div></td><td style=\"white-space: nowrap;\" class=\"PollCount\">"));

        // Create lable with answer count
        if (this.CountType == CountTypeEnum.Absolute)
        {
            // Absolute count
            pnlAnswer.Controls.Add(new LiteralControl(currentValue.ToString()));
        }
        else if (this.CountType == CountTypeEnum.Percentage)
        {
            // Percentage count
            int percent = 0;
            if (countSummary != 0)
            {
                percent = (int)Math.Round(((double)currentValue / countSummary) * 100);
            }
            pnlAnswer.Controls.Add(new LiteralControl(percent.ToString() + "%"));
        }

        // End PollAnswerGraph
        pnlAnswer.Controls.Add(new LiteralControl("</td></tr>"));
    }



    /// <summary>
    /// On btnVote click event handler.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnVote_OnClick(object sender, EventArgs e)
    {
        if (pi != null)
        {
            // Indicates whether user voted or not
            bool voted = false;

            // Check if user has already voted
            if ((this.CheckVoted) && (PollInfoProvider.HasVoted(pi.PollID)))
            {
                errMessage = ResHelper.GetString("Polls.UserHasVoted");
            }
            else if (isOpened)
            {
                // Get poll answers
                DataSet ds = PollAnswerInfoProvider.GetAnswers(pi.PollID);
                if (!DataHelper.DataSourceIsEmpty(ds))
                {
                    DataRowCollection rows = ds.Tables[0].Rows;

                    LocalizedCheckBox chkItem = null;
                    LocalizedRadioButton radItem = null;

                    foreach (DataRow row in rows)
                    {
                        PollAnswerInfo pai = new PollAnswerInfo(row);

                        if ((pai != null) && (pai.AnswerEnabled))
                        {
                            // Find specific controls and update pollanswerinfo if controls are checked
                            if (pi.PollAllowMultipleAnswers)
                            {
                                // Find checkbox
                                chkItem = (LocalizedCheckBox)this.pnlAnswer.FindControl("chk" + pai.AnswerID);

                                if ((chkItem != null) && (chkItem.Checked))
                                {
                                    PollAnswerInfoProvider.Vote(pai.AnswerID);
                                    voted = true;
                                }
                            }
                            else
                            {
                                // Find radiobutton
                                radItem = (LocalizedRadioButton)this.pnlAnswer.FindControl("rad" + pai.AnswerID);

                                if ((radItem != null) && (radItem.Checked))
                                {
                                    PollAnswerInfoProvider.Vote(pai.AnswerID);
                                    voted = true;
                                }
                            }
                        }
                    }

                    if ((this.CheckVoted) && (voted))
                    {
                        // Create cookie about user's voting
                        PollInfoProvider.SetVoted(pi.PollID);
                    }
                }
            }

            if (voted)
            {
                // Reload poll control
                ReloadData(true);
            }
            else
            {
                lblInfo.Text = ResHelper.GetString("Polls.DidNotVoted");
            }
        }
    }


    /// <summary>
    /// Return true if user has permissions.
    /// </summary>
    protected bool HasPermission()
    {
        if ((!this.CheckPermissions) || (CMSContext.CurrentUser.IsGlobalAdministrator))
        {
            return true;
        }

        bool toReturn = false;

        if (pi != null)
        {
            // Access to all users
            if (pi.PollAccess == PollInfoProvider.ACCESS_ALL)
            {
                toReturn = true;
            }

            // Access to authenticated user
            if ((pi.PollAccess == PollInfoProvider.ACCESS_AUTHENTICATED) && (!CMSContext.CurrentUser.IsPublic()))
            {
                toReturn = true;
            }

            // Access to roles
            if ((pi.PollAccess == PollInfoProvider.ACCESS_ROLES) && (CMSContext.CurrentUser != null))
            {
                DataTable roles = UserInfoProvider.GetUserRoles(CMSContext.CurrentUser.UserID);

                string role = "";
                foreach (DataRow row in roles.Rows)
                {
                    role = ValidationHelper.GetString(row["RoleName"], "").ToLower();
                    if (pi.AllowedRoles.Contains(role) && RoleInfoProvider.RoleExists(role, CMSContext.CurrentSite.SiteName))
                    {
                        toReturn = true;
                    }
                }
            }
        }

        return toReturn;
    }


    /// <summary>
    /// Return true if actual time is inside the poll's opening time or the poll's open from/to are not set.
    /// </summary>
    protected bool IsOpened()
    {
        if (!this.CheckOpen)
        {
            return true;
        }

        bool toReturn = false;

        if (pi != null)
        {
            DateTime now = DateTime.Now;
            if ((pi.PollOpenFrom == DataHelper.DATETIME_NOT_SELECTED) && (pi.PollOpenTo == DataHelper.DATETIME_NOT_SELECTED))
            {
                toReturn = true;
            }
            else if ((pi.PollOpenFrom != DataHelper.DATETIME_NOT_SELECTED) && (pi.PollOpenTo == DataHelper.DATETIME_NOT_SELECTED) && (pi.PollOpenFrom < now))
            {
                toReturn = true;
            }
            else if ((pi.PollOpenFrom == DataHelper.DATETIME_NOT_SELECTED) && (pi.PollOpenTo != DataHelper.DATETIME_NOT_SELECTED) && (pi.PollOpenTo > now))
            {
                toReturn = true;
            }
            else if ((pi.PollOpenFrom != DataHelper.DATETIME_NOT_SELECTED) && (pi.PollOpenTo != DataHelper.DATETIME_NOT_SELECTED) && (pi.PollOpenFrom < now) && (pi.PollOpenTo > now))
            {
                toReturn = true;
            }
        }

        return toReturn;
    }

}
